//
//  BsKVOHelper.swift
//  BaiSiSMApp
//
//  Created by davidhuang on 2022/10/19.
//  Copyright © 2022 www.davidhuang.com. All rights reserved.
//

import UIKit

///解决重复添加；解决重复移除；解决未添加却移除
open class BsKVOHelper {
    
    class Container: Equatable {
        static func == (lhs: BsKVOHelper.Container, rhs: BsKVOHelper.Container) -> Bool {
            guard let lObserver = lhs.observer,
                let lObject = lhs.object,
                let rObserver = rhs.observer,
                let rObject = rhs.object else {
                return false
            }
            if lObserver == rObserver, lObject == rObject{
                return lhs.keyPath == rhs.keyPath
            }
            return false
        }
        
        var observer: Int?
        var object: Int?
        var keyPath: String
        
        init(observer: Int?, object: Int?, keyPath: String) {
            self.observer = observer
            self.object = object
            self.keyPath = keyPath
        }
        
        func clear(){
            observer = nil
            object = nil
        }
    }

    private var containers: [Container] = []
    
    public init(){
        
    }
    
    open func add(observer: NSObject,toObject object: NSObject,forKeyPath keyPath: String, options: NSKeyValueObservingOptions = [], context: UnsafeMutableRawPointer?){
        let aContainer = Container(observer: observer.hashValue, object: object.hashValue, keyPath: keyPath)
        if containers.contains(where: { (thisContainer) -> Bool in
            return aContainer == thisContainer
        }){
            return
        }
        containers.append(aContainer)
        object.addObserver(observer,forKeyPath: keyPath, options: options, context: context)
    }
    
    open func remove(observer: NSObject,atObject object: NSObject,forKeyPath keyPath: String){
        let aContainer = Container(observer: observer.hashValue, object: object.hashValue, keyPath: keyPath)
        if let first = containers.first(where: { (thisContainer) -> Bool in
             return aContainer == thisContainer
        }){
            object.removeObserver(observer, forKeyPath: keyPath)
            first.clear()
        }
    }
}
