//
//  BskWebViewPreloadManager.swift
//  BaiSiSMApp
//
//  Created by davidhuang on 2022/11/22.
//  Copyright © 2022 www.davidhuang.com. All rights reserved.
//

import UIKit
import WebKit

class BsWebViewPreloadManager : NSObject {
    private var dicPreloadedViews : [String : AnyObject] = [:]

    static let share = BsWebViewPreloadManager()

    override init() {
        super.init()
        addPreloadingView()
    }



//    /**
//     根据URL 预初始化若干WebView
//     */
//    - (void)addPreloadingView:(NSArray *)urls {
//        for (NSString *url in urls) {
//            if (![self.urls containsObject:url]) {
//                [self.urls addObject: url];
//                ReuseWebView *webView = [self createWebView];
//                [webView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:url]]];
//                [self.preloadedViews addEntriesFromDictionary:@{url:webView}];
//            }
//        }
//    }


}


// MARK: - 私有方法 methods
extension BsWebViewPreloadManager {


    private func getWebview(urlString : String) -> WKWebView {
        if let wk = dicPreloadedViews[urlString] {
            return wk as! WKWebView
        }
        return createWebView(urlString: urlString)
    }

    private func createWebView(urlString : String) -> WKWebView {

        let jScript = "var meta = document.createElement('meta'); meta.setAttribute('name', 'viewport'); meta.setAttribute('content', 'width=device-width'); document.getElementsByTagName('head')[0].appendChild(meta);"
        let userScript = WKUserScript(source: jScript, injectionTime: WKUserScriptInjectionTime.atDocumentEnd, forMainFrameOnly: true)

        let userContentController = WKUserContentController()
        userContentController.addUserScript(userScript)


        let cfg = WKWebViewConfiguration()
        cfg.allowsInlineMediaPlayback = true
        cfg.mediaTypesRequiringUserActionForPlayback = []
        cfg.userContentController = userContentController

        let wv = WKWebView(frame: UIScreen.main.bounds,configuration: cfg)
        if let nsURL = URL(string: urlString){
            let request = URLRequest(url: nsURL)
            wv.load(request)
            return wv
        }
        return WKWebView()
    }


#if DEBUG
    func onlyCreateWebView() -> WKWebView {
        let jScript = "var meta = document.createElement('meta'); meta.setAttribute('name', 'viewport'); meta.setAttribute('content', 'width=device-width'); document.getElementsByTagName('head')[0].appendChild(meta);"
        let userScript = WKUserScript(source: jScript, injectionTime: WKUserScriptInjectionTime.atDocumentEnd, forMainFrameOnly: true)

        let userContentController = WKUserContentController()
        userContentController.addUserScript(userScript)


        let cfg = WKWebViewConfiguration()
        cfg.allowsInlineMediaPlayback = true
        cfg.mediaTypesRequiringUserActionForPlayback = []
        cfg.userContentController = userContentController

        let wv = WKWebView(frame: UIScreen.main.bounds,configuration: cfg)
        return wv
    }
#endif

}

// MARK: - public methods
extension BsWebViewPreloadManager {
    func addPreloadingView()  {
        let arrUrl = [
            BsConstant.URL.medical_center_url]

        for item in arrUrl {
            let wv = createWebView(urlString: item)
            dicPreloadedViews[item] = wv
        }
    }
    
    func getWebView(urlString : String) -> WKWebView {
        let urls = dicPreloadedViews.keys
        if urls.contains(urlString) == false {
            return getWebview(urlString: urlString)
        } else {
            let wk = createWebView(urlString: urlString)
            dicPreloadedViews[urlString] = wk
            return wk
        }
    }
}
